import typer
from InquirerPy.base.control import Choice
from InquirerPy.prompts.list import ListPrompt
from InquirerPy.prompts.fuzzy import FuzzyPrompt
from InquirerPy.prompts.confirm import ConfirmPrompt
from typing_extensions import Annotated, Optional

from .command import Command
from .sessions import get_session_choices
from .ssh import parse_ssh_configurations, ssh_connect
from .utils import is_vscode_available


app = typer.Typer()

def autocompletion(incomplete: str):
    completion = []
    config = parse_ssh_configurations()
    hostnames = [name for name in config.get_hostnames() if name != "*"]
    for name in hostnames:
        if name.startswith(incomplete):
            completion.append(name)
    return completion

@app.command()
def main(hostname_or_alias: Annotated[
    Optional[str], 
    typer.Argument(help="", autocompletion=autocompletion)
    ] = None
):
    client = None
    try:
        command = Command()

        # ssh
        config = parse_ssh_configurations()
        if hostname_or_alias is None:
            hostnames = [name for name in config.get_hostnames() if name != "*"]
            hostname_or_alias = FuzzyPrompt(
                message="Select host to connect:",
                choices=list(hostnames),
            ).execute()
            assert hostname_or_alias is not None
        client = ssh_connect(hostname_or_alias, config)
        command.add_ssh_command(hostname_or_alias, client)

        # docker
        _, stdout, _ = client.exec_command('docker ps --format "{{.Names}}"')
        docker_container_names = list(
            filter(None, stdout.read().decode().strip().split("\n"))
        ) + [Choice(value=None, name="Host machine without docker")]
        command.add_docker_command(FuzzyPrompt(
            message="Select docker container to connect:",
            choices=docker_container_names,
        ).execute())

        # vscode
        if is_vscode_available():
            command.add_vscode_launch(ConfirmPrompt(
                "Launch VS Code?",
                default=True,
                confirm_letter="y",
                reject_letter="n",
            ).execute())

        # session
        session_choices = get_session_choices(
            command.is_tmux_available_on_remote()
        )
        command.add_session_command(ListPrompt(
            message="Select session to connect:",
            choices=session_choices,
            default=session_choices[-1].value
        ).execute())
    except KeyboardInterrupt:
        if client:
            client.close()
        exit(1)

    # exec
    client.close()
    command.exec() 
    exit(1)


if __name__ == "__main__":
    typer.run(main)


import os
import getpass

from paramiko import SSHConfig, SSHClient, ProxyCommand, RejectPolicy
from paramiko.ssh_exception import AuthenticationException

def parse_ssh_configurations() -> SSHConfig:
    home_directory = os.path.expanduser("~")
    ssh_config_path = os.getenv(
        "SSSH_SSH_CONFIG", 
        os.path.join(home_directory, ".ssh", "config")
    )

    return SSHConfig.from_path(ssh_config_path)


def ssh_connect(hostname_or_alias: str, config: SSHConfig) -> SSHClient:
    cfg = config.lookup(hostname_or_alias)
    hostname = cfg.get("hostname", hostname_or_alias)
    username = cfg.get("user")
    port = int(cfg.get("port", 22))
    key_files = cfg.get("identityfile", [None])[0]
    proxy_cmd = cfg.get("proxycommand")

    sock = None
    if proxy_cmd:
        sock = ProxyCommand(proxy_cmd)

    client = SSHClient()
    client.load_system_host_keys()
    client.set_missing_host_key_policy(RejectPolicy())

    try:
        client.connect(
            hostname=hostname,
            port=port,
            username=username,
            key_filename=key_files,
            sock=sock,
            allow_agent=True,
            look_for_keys=True,
        )
    except AuthenticationException as e:
        password = getpass.getpass(f"{username}@{hostname}'s password:")
        client.connect(
            hostname=hostname,
            password=password,
            port=port,
            username=username,
            key_filename=key_files,
            sock=sock,
            allow_agent=True,
            look_for_keys=True,
        )

    return client




from enum import Enum

from InquirerPy.base.control import Choice

from .utils import is_iterm

class Session(Enum):
    TMUX_ITERM2 = "tmux -CC -u new -A -s main"
    TMUX = "tmux new -A -s main"
    SSH = "/bin/bash"


def get_session_choices(is_tmux_available: bool) -> list[Choice]:
    session_choices = [
        Choice(
            value=Session.SSH.value, 
            name="ssh session without tmux (default shell)"
        ),
    ]
    if is_tmux_available:
        session_choices.append(
            Choice(
                value=Session.TMUX.value, 
                name="use tmux (tmux)"
            )
        )
    if is_iterm():
        session_choices.append(
            Choice(
                value=Session.TMUX_ITERM2.value, 
                name="use tmux with iterm2 integration (tmux -CC)"
            )
        )   
    return session_choices


import os
import psutil


def is_iterm() -> bool:
    if os.environ.get("TERM_PROGRAM") == "iTerm.app":
        return True

    for key in ("ITERM_SESSION_ID", "ITERM_PROFILE", "ITERM2_SESSION_ID"):
        if key in os.environ:
            return True
    
    try:
        parent = psutil.Process().parent()
        while parent:
            if "iTerm" in parent.name():
                return True
            parent = parent.parent()
    except Exception:
        pass

    return False

def is_vscode_available() -> bool:
    return os.system("which code > /dev/null 2>&1") == 0



import os
import pexpect
import shlex
import json

from paramiko import SSHClient

from .sessions import Session

class Command():
    def __init__(self) -> None:
        # ssh
        self.username = None
        self.hostname = None
        self.password = None

        # docker
        self.container_name = None

        # session
        self.session = None

    def add_ssh_command(self, hostname_or_alias: str, client: SSHClient):
        transport = client.get_transport()
        assert transport and transport.auth_handler  

        self.hostname = hostname_or_alias
        self.username = transport.auth_handler.get_username()
        self.password = transport.auth_handler.password

    def add_docker_command(self, container_name: str | None):
        self.container_name = container_name

    def add_session_command(self, session: str):
        self.session = session
        if self.container_name and session == Session.SSH.value:
            # TODO: fix this
            self.session = "/bin/bash"

    def is_tmux_available_on_remote(self) -> bool:
        # TODO: implement this
        return True

    def add_vscode_launch(self, enable: bool):
        self.launch_vscode = enable

    def exec(self):
        if self.launch_vscode:
            vscode_cmd = "code -n --remote"
            if self.container_name:
                json_str = json.dumps({
                    "settings": {
                        "host": f"ssh://{self.username}@{self.hostname}",
                    },
                    "containerName": self.container_name,
                }, separators=(",", ":"))
                hex_str = "".join(f"{ord(ch):02x}" for ch in json_str)
                vscode_cmd += f' "attached-container+{hex_str}"'
            else:
                vscode_cmd += f' "ssh-remote+{self.username}@{self.hostname}"'
            os.system(vscode_cmd)

        cmd = f"ssh -t {self.username}@{self.hostname}"
        if self.container_name:
            cmd += f" docker exec -it {self.container_name}"
        cmd += f" {self.session}"
        if self.password:
            child = pexpect.spawn(cmd, encoding="utf-8")
            child.expect("password:")
            child.sendline(self.password)
            child.interact()
            child.close()
        else:
            os.execvp("ssh", shlex.split(cmd))
